require 'redmine'

Redmine::Plugin.register :redmine_testlink do
  name 'TestLink plugin for Redmine'
  author 'SundayWalker'
  description 'This is a TestLink plugin for Redmine'
  version '0.0.1'

  settings :default => {
    'testlink_base_path' => '/testlink'
  }, :partial => 'settings/testlink_settings'

  project_module :testlink do
    permission :testlink, {:testlink => [:index]}, :public => false
  end

  menu :project_menu, :testlink, { :controller => 'testlink', :action => 'index' }, :caption => :testlink_caption, :param => :id
end


Redmine::WikiFormatting::Macros.register do
  desc "Link to TestLink macro \n"+
       "!{{linkto_testlink(arg)}}\n\n"+
       "Examples:\n"+
       "!{{linkto_testlink}}                                               -- TestLink home\n"+
       "!{{linkto_testlink(reqSpecMgmt)}}                                  -- requrement\n"+
       "!{{linkto_testlink(editTc)}}                                       -- edit testcase\n"+
       "!{{linkto_testlink(editTc&edit=testcase&id=4)}}\n"+
       "!{{linkto_testlink(editTc&edit=testsuite&id=1)}}\n"+
       "!{{linkto_testlink(editTc&edit=testproject&id=1)}}\n"+
       "!{{linkto_testlink(executeTest)}}                                  -- execute test\n"+
       "!{{linkto_testlink(executeTest&id=4&build_id=5)}}\n"+
       "!{{linkto_testlink(showMetrics)}}                                  -- show metrics\n"+
       "!{{linkto_testlink(testplan&format=0&tplan_id=10)}}                -- Test Plan Report\n"+
       "!{{linkto_testlink(testreport&format=0&tplan_id=10)}}              -- Test Report\n"+
       "!{{linkto_testlink(resultsGeneral&format=0&tplan_id=10)}}          -- General Test Plan Metrics\n"+
       "!{{linkto_testlink(resultsByTesterPerBuild&format=0&tplan_id=10)}} -- Results by Tester per Build\n"+
       "!{{linkto_testlink(tcAssignedToUser&format=0&tplan_id=10)}}        -- Test Case Assignment Overview\n"+
       "!{{linkto_testlink(resultsMoreBuildsGUI&format=0&tplan_id=10)}}    -- Query Metrics\n"+
       "!{{linkto_testlink(resultsTC&format=0&tplan_id=10)}}               -- Test result matrix\n"+
       "!{{linkto_testlink(f&format=0&tplan_id=10)}}                       -- Failed Test Cases\n"+
       "!{{linkto_testlink(b&format=0&tplan_id=10)}}                       -- Blocked Test Cases\n"+
       "!{{linkto_testlink(n&format=0&tplan_id=10)}}                       -- Not run Test Cases\n"+
       "!{{linkto_testlink(testCasesWithoutTester&format=0&tplan_id=10)}}  -- Test Cases without Tester Assignment\n"+
       "!{{linkto_testlink(charts&format=0&tplan_id=10)}}                  -- Charts\n"+
       "!{{linkto_testlink(resultsReqs&format=0&tplan_id=10)}}             -- Requirements based Report\n"+
       "!{{linkto_testlink(uncoveredTestCases&format=0&tplan_id=10)}}      -- Test Cases without Requirements Assignment\n"+
       "!{{linkto_testlink(testCasesWithCF&format=0&tplan_id=10)}}         -- Test Cases with Custom Fields info\n"+
       "!{{linkto_testlink(testPlanWithCF&format=0&tplan_id=10)}}          -- Test Plan with Custom Field info\n"+
       "!{{linkto_testlink(freeTestCases&format=0&tplan_id=10)}}           -- Test Cases not assigned to Any Test Plan\n"+
       ""

  macro :linkto_testlink do |obj, args|
    url = url_for(:controller => 'testlink', :action => 'index')
    if  args.size > 0 then
      ahref = content_tag("a", "testlink:"+args[0].strip, :href => url+"?feature="+args[0].strip.gsub(/(\W)format(\W)/, '\1oformat\2').gsub(/(\W)id(\W)/, '\1testcase_id\2'))
    else
      ahref = content_tag("a", "testlink:", :href => url)
    end
    ahref
  end
end

Redmine::WikiFormatting::Macros.register do
  desc "Include TestLink macro - Auto fitting height \n"+
       "!{{include_testlink(arg[,height[,width]])}}\n\n"+
       "Examples:\n"+
       "!{{include_testlink(editTc&edit=testcase&id=4)}}                    -- edit testcase\n"+
       "!{{include_testlink(editTc&edit=testsuite&id=1)}}\n"+
       "!{{include_testlink(editTc&edit=testproject&id=1)}}\n"+
       "!{{include_testlink(executeTest&id=4&build_id=5)}}                  -- execute test\n"+
       "!{{include_testlink(testplan&format=0&tplan_id=10)}}                -- Test Plan Report\n"+
       "!{{include_testlink(testreport&format=0&tplan_id=10)}}              -- Test Report\n"+
       "!{{include_testlink(resultsGeneral&format=0&tplan_id=10)}}          -- General Test Plan Metrics\n"+
       "!{{include_testlink(resultsByTesterPerBuild&format=0&tplan_id=10)}} -- Results by Tester per Build\n"+
       "!{{include_testlink(tcAssignedToUser&format=0&tplan_id=10)}}        -- Test Case Assignment Overview\n"+
       "!{{include_testlink(resultsMoreBuildsGUI&format=0&tplan_id=10)}}    -- Query Metrics\n"+
       "!{{include_testlink(resultsTC&format=0&tplan_id=10)}}               -- Test result matrix\n"+
       "!{{include_testlink(f&format=0&tplan_id=10)}}                       -- Failed Test Cases\n"+
       "!{{include_testlink(b&format=0&tplan_id=10)}}                       -- Blocked Test Cases\n"+
       "!{{include_testlink(n&format=0&tplan_id=10)}}                       -- Not run Test Cases\n"+
       "!{{include_testlink(testCasesWithoutTester&format=0&tplan_id=10)}}  -- Test Cases without Tester Assignment\n"+
       "!{{include_testlink(charts&format=0&tplan_id=10)}}                  -- Charts\n"+
       "!{{include_testlink(resultsReqs&format=0&tplan_id=10)}}             -- Requirements based Report\n"+
       "!{{include_testlink(uncoveredTestCases&format=0&tplan_id=10)}}      -- Test Cases without Requirements Assignment\n"+
       "!{{include_testlink(testCasesWithCF&format=0&tplan_id=10)}}         -- Test Cases with Custom Fields info\n"+
       "!{{include_testlink(testPlanWithCF&format=0&tplan_id=10)}}          -- Test Plan with Custom Field info\n"+
       "!{{include_testlink(freeTestCases&format=0&tplan_id=10)}}           -- Test Cases not assigned to Any Test Plan\n"+
       ""

  macro :include_testlink do |obj, args|
#   if (true) then
   if (false) then
    @testlink_base_path = Setting.plugin_redmine_testlink['testlink_base_path']
    params['feature']    = ''

    params['edit'] = ''
    params['testcase_id'] = ''
    params['build_id'] = ''
    params['version_id'] = ''
    params['tplan_id'] = ''
    params['format'] = ''
    params['docTestPlanId'] = ''
    params['toc'] = ''
    params['headerNumbering'] = ''
    params['header'] = ''
    params['summary'] = ''
    params['body'] = ''
    params['author'] = ''
    params['keyword'] = ''
    params['cfields'] = ''
    params['requirement'] = ''
    params['testplan'] = ''
    params['level'] = ''

    params['feature']    = args[0].sub(/(^)(\w*)(\W.*$|$)/, '\2') if /(^)(\w*)(\W|$)/ =~ args[0]

    params['edit'] = args[0].sub(/^.*(\W)edit\W(\w*)(\W.*$|$)/, '\2') if /(\W)edit\W(\w*)(\W|$)/ =~ args[0]
    params['testcase_id'] = args[0].sub(/^.*(\W)id\W(\w*)(\W.*$|$)/, '\2') if /(\W)id\W(\w*)(\W|$)/ =~ args[0]
    params['testcase_id'] = args[0].sub(/^.*(\W)testcase_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)testcase_id\W(\w*)(\W|$)/ =~ args[0]
    params['build_id'] = args[0].sub(/^.*(\W)build_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)build_id\W(\w*)(\W|$)/ =~ args[0]
    params['version_id'] = args[0].sub(/^.*(\W)version_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)version_id\W(\w*)(\W|$)/ =~ args[0]
    params['tplan_id'] = args[0].sub(/^.*(\W)tplan_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)tplan_id\W(\w*)(\W|$)/ =~ args[0]
    params['format'] = args[0].sub(/^.*(\W)format\W(\w*)(\W.*$|$)/, '\2') if /(\W)format\W(\w*)(\W|$)/ =~ args[0]
    params['docTestPlanId'] = args[0].sub(/^.*(\W)docTestPlanId\W(\w*)(\W.*$|$)/, '\2') if /(\W)docTestPlanId\W(\w*)(\W|$)/ =~ args[0]
    params['toc'] = args[0].sub(/^.*(\W)toc\W(\w*)(\W.*$|$)/, '\2') if /(\W)toc\W(\w*)(\W|$)/ =~ args[0]
    params['headerNumbering'] = args[0].sub(/^.*(\W)headerNumbering\W(\w*)(\W.*$|$)/, '\2') if /(\W)headerNumbering\W(\w*)(\W|$)/ =~ args[0]
    params['header'] = args[0].sub(/^.*(\W)header\W(\w*)(\W.*$|$)/, '\2') if /(\W)header\W(\w*)(\W|$)/ =~ args[0]
    params['summary'] = args[0].sub(/^.*(\W)summary\W(\w*)(\W.*$|$)/, '\2') if /(\W)summary\W(\w*)(\W|$)/ =~ args[0]
    params['body'] = args[0].sub(/^.*(\W)body\W(\w*)(\W.*$|$)/, '\2') if /(\W)body\W(\w*)(\W|$)/ =~ args[0]
    params['author'] = args[0].sub(/^.*(\W)author\W(\w*)(\W.*$|$)/, '\2') if /(\W)author\W(\w*)(\W|$)/ =~ args[0]
    params['keyword'] = args[0].sub(/^.*(\W)keyword\W(\w*)(\W.*$|$)/, '\2') if /(\W)keyword\W(\w*)(\W|$)/ =~ args[0]
    params['cfields'] = args[0].sub(/^.*(\W)cfields\W(\w*)(\W.*$|$)/, '\2') if /(\W)cfields\W(\w*)(\W|$)/ =~ args[0]
    params['requirement'] = args[0].sub(/^.*(\W)requirement\W(\w*)(\W.*$|$)/, '\2') if /(\W)requirement\W(\w*)(\W|$)/ =~ args[0]
    params['testplan'] = args[0].sub(/^.*(\W)testplan\W(\w*)(\W.*$|$)/, '\2') if /(\W)testplan\W(\w*)(\W|$)/ =~ args[0]
    params['notes'] = args[0].sub(/^.*(\W)notes\W(\w*)(\W.*$|$)/, '\2') if /(\W)notes\W(\w*)(\W|$)/ =~ args[0]
    params['passfail'] = args[0].sub(/^.*(\W)passfail\W(\w*)(\W.*$|$)/, '\2') if /(\W)passfail\W(\w*)(\W|$)/ =~ args[0]
    params['metrics'] = args[0].sub(/^.*(\W)metrics\W(\w*)(\W.*$|$)/, '\2') if /(\W)metrics\W(\w*)(\W|$)/ =~ args[0]
    params['level'] = args[0].sub(/^.*(\W)level\W(\w*)(\W.*$|$)/, '\2') if /(\W)level\W(\w*)(\W|$)/ =~ args[0]

    ta  = TestlinkAdapter.new

    project_id=obj.project.identifier # obj.project.id.to_s
    projectname=obj.project.name

    arg = "var project_id='"+project_id+"';\nvar projectname='"+projectname+"';\n"
    arg = arg+ta.getTestlinkParams(params)

    url = ta.getTestlinkUrl(@testlink_base_path, params)
    frame_number = rand(100000000).to_s
    frame_fname  = 'ResizeIframe'+frame_number
    frame_id     = 'frame'+frame_number
    frame_name   = frame_id
    ahref = content_tag("iframe", "testlink:"+args[0].strip, :src => url, :width => '100%', :id => frame_id, :name => frame_name, :onload => frame_fname+'(this.id,this.name)')

    div_id = 'message'+rand(100000000).to_s

    arg = arg+ta.getTestprojectId('$("'+div_id+'").innerHTML = '+"'"+ahref+"';", div_id)
    arg = arg+ta.getResizeIframe(frame_fname)
    rtn = ta.getJavascript(arg, div_id).gsub(/div/, 'span')
   else
    ta  = TestlinkAdapter.new
    rtn = ta.getTestlinkIframe(obj, args)
   end
    rtn
  end

end

Redmine::WikiFormatting::Macros.register do
  desc "Iframe TestLink macro \n"+
       "!{{iframe_testlink(arg[,height[,width]])}}\n\n"+
       "Examples:\n"+
       "!{{iframe_testlink(editTc&edit=testcase&id=4)}}                    -- edit testcase\n"+
       "!{{iframe_testlink(editTc&edit=testsuite&id=1)}}\n"+
       "!{{iframe_testlink(editTc&edit=testproject&id=1)}}\n"+
       "!{{iframe_testlink(executeTest)}}                                  -- execute test\n"+
       "!{{iframe_testlink(executeTest&id=4&build_id=5)}}\n"+
       "!{{iframe_testlink(showMetrics)}}                                  -- show metrics\n"+
       "!{{iframe_testlink(testplan&format=0&tplan_id=10)}}                -- Test Plan Report\n"+
       "!{{iframe_testlink(testreport&format=0&tplan_id=10)}}              -- Test Report\n"+
       "!{{iframe_testlink(resultsGeneral&format=0&tplan_id=10)}}          -- General Test Plan Metrics\n"+
       "!{{iframe_testlink(resultsByTesterPerBuild&format=0&tplan_id=10)}} -- Results by Tester per Build\n"+
       "!{{iframe_testlink(tcAssignedToUser&format=0&tplan_id=10)}}        -- Test Case Assignment Overview\n"+
       "!{{iframe_testlink(resultsMoreBuildsGUI&format=0&tplan_id=10)}}    -- Query Metrics\n"+
       "!{{iframe_testlink(resultsTC&format=0&tplan_id=10)}}               -- Test result matrix\n"+
       "!{{iframe_testlink(f&format=0&tplan_id=10)}}                       -- Failed Test Cases\n"+
       "!{{iframe_testlink(b&format=0&tplan_id=10)}}                       -- Blocked Test Cases\n"+
       "!{{iframe_testlink(n&format=0&tplan_id=10)}}                       -- Not run Test Cases\n"+
       "!{{iframe_testlink(testCasesWithoutTester&format=0&tplan_id=10)}}  -- Test Cases without Tester Assignment\n"+
       "!{{iframe_testlink(charts&format=0&tplan_id=10)}}                  -- Charts\n"+
       "!{{iframe_testlink(resultsReqs&format=0&tplan_id=10)}}             -- Requirements based Report\n"+
       "!{{iframe_testlink(uncoveredTestCases&format=0&tplan_id=10)}}      -- Test Cases without Requirements Assignment\n"+
       "!{{iframe_testlink(testCasesWithCF&format=0&tplan_id=10)}}         -- Test Cases with Custom Fields info\n"+
       "!{{iframe_testlink(testPlanWithCF&format=0&tplan_id=10)}}          -- Test Plan with Custom Field info\n"+
       "!{{iframe_testlink(freeTestCases&format=0&tplan_id=10)}}           -- Test Cases not assigned to Any Test Plan\n"+
       ""

  macro :iframe_testlink do |obj, args|
#   if (true) then
   if (false) then
    @testlink_base_path = Setting.plugin_redmine_testlink['testlink_base_path']
    params['feature']    = ''

    params['edit'] = ''
    params['testcase_id'] = ''
    params['build_id'] = ''
    params['version_id'] = ''
    params['tplan_id'] = ''
    params['format'] = ''
    params['docTestPlanId'] = ''
    params['toc'] = ''
    params['headerNumbering'] = ''
    params['header'] = ''
    params['summary'] = ''
    params['body'] = ''
    params['author'] = ''
    params['keyword'] = ''
    params['cfields'] = ''
    params['requirement'] = ''
    params['testplan'] = ''
    params['level'] = ''

    params['feature']    = args[0].sub(/(^)(\w*)(\W.*$|$)/, '\2') if /(^)(\w*)(\W|$)/ =~ args[0]

    params['edit'] = args[0].sub(/^.*(\W)edit\W(\w*)(\W.*$|$)/, '\2') if /(\W)edit\W(\w*)(\W|$)/ =~ args[0]
    params['testcase_id'] = args[0].sub(/^.*(\W)id\W(\w*)(\W.*$|$)/, '\2') if /(\W)id\W(\w*)(\W|$)/ =~ args[0]
    params['testcase_id'] = args[0].sub(/^.*(\W)testcase_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)testcase_id\W(\w*)(\W|$)/ =~ args[0]
    params['build_id'] = args[0].sub(/^.*(\W)build_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)build_id\W(\w*)(\W|$)/ =~ args[0]
    params['version_id'] = args[0].sub(/^.*(\W)version_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)version_id\W(\w*)(\W|$)/ =~ args[0]
    params['tplan_id'] = args[0].sub(/^.*(\W)tplan_id\W(\w*)(\W.*$|$)/, '\2') if /(\W)tplan_id\W(\w*)(\W|$)/ =~ args[0]
    params['format'] = args[0].sub(/^.*(\W)format\W(\w*)(\W.*$|$)/, '\2') if /(\W)format\W(\w*)(\W|$)/ =~ args[0]
    params['docTestPlanId'] = args[0].sub(/^.*(\W)docTestPlanId\W(\w*)(\W.*$|$)/, '\2') if /(\W)docTestPlanId\W(\w*)(\W|$)/ =~ args[0]
    params['toc'] = args[0].sub(/^.*(\W)toc\W(\w*)(\W.*$|$)/, '\2') if /(\W)toc\W(\w*)(\W|$)/ =~ args[0]
    params['headerNumbering'] = args[0].sub(/^.*(\W)headerNumbering\W(\w*)(\W.*$|$)/, '\2') if /(\W)headerNumbering\W(\w*)(\W|$)/ =~ args[0]
    params['header'] = args[0].sub(/^.*(\W)header\W(\w*)(\W.*$|$)/, '\2') if /(\W)header\W(\w*)(\W|$)/ =~ args[0]
    params['summary'] = args[0].sub(/^.*(\W)summary\W(\w*)(\W.*$|$)/, '\2') if /(\W)summary\W(\w*)(\W|$)/ =~ args[0]
    params['body'] = args[0].sub(/^.*(\W)body\W(\w*)(\W.*$|$)/, '\2') if /(\W)body\W(\w*)(\W|$)/ =~ args[0]
    params['author'] = args[0].sub(/^.*(\W)author\W(\w*)(\W.*$|$)/, '\2') if /(\W)author\W(\w*)(\W|$)/ =~ args[0]
    params['keyword'] = args[0].sub(/^.*(\W)keyword\W(\w*)(\W.*$|$)/, '\2') if /(\W)keyword\W(\w*)(\W|$)/ =~ args[0]
    params['cfields'] = args[0].sub(/^.*(\W)cfields\W(\w*)(\W.*$|$)/, '\2') if /(\W)cfields\W(\w*)(\W|$)/ =~ args[0]
    params['requirement'] = args[0].sub(/^.*(\W)requirement\W(\w*)(\W.*$|$)/, '\2') if /(\W)requirement\W(\w*)(\W|$)/ =~ args[0]
    params['testplan'] = args[0].sub(/^.*(\W)testplan\W(\w*)(\W.*$|$)/, '\2') if /(\W)testplan\W(\w*)(\W|$)/ =~ args[0]
    params['notes'] = args[0].sub(/^.*(\W)notes\W(\w*)(\W.*$|$)/, '\2') if /(\W)notes\W(\w*)(\W|$)/ =~ args[0]
    params['passfail'] = args[0].sub(/^.*(\W)passfail\W(\w*)(\W.*$|$)/, '\2') if /(\W)passfail\W(\w*)(\W|$)/ =~ args[0]
    params['metrics'] = args[0].sub(/^.*(\W)metrics\W(\w*)(\W.*$|$)/, '\2') if /(\W)metrics\W(\w*)(\W|$)/ =~ args[0]
    params['level'] = args[0].sub(/^.*(\W)level\W(\w*)(\W.*$|$)/, '\2') if /(\W)level\W(\w*)(\W|$)/ =~ args[0]

    ta  = TestlinkAdapter.new

    project_id=obj.project.identifier # obj.project.id.to_s
    projectname=obj.project.name

    arg = "var project_id='"+project_id+"';\nvar projectname='"+projectname+"';\n"
    arg = arg+ta.getTestlinkParams(params)

    url = ta.getTestlinkUrl(@testlink_base_path, params, true)
    frame_number = rand(100000000).to_s
#    frame_fname  = 'ResizeIframe'+frame_number
    frame_fname  = 'ChangeIframe'+frame_number
    frame_id     = 'frame'+frame_number
    frame_name   = frame_id
    ahref = content_tag("iframe", "testlink:"+args[0].strip, :src => url, :width => '100%', :id => frame_id, :name => frame_name, :onload => frame_fname+'(this.id,this.name)')

#    div_id = 'message'+rand(100000000).to_s
    div_id = 'message'+frame_number

    arg = arg+ta.getTestprojectId('$("'+div_id+'").innerHTML = '+"'"+ahref+"';", div_id)
#    arg = arg+ta.getResizeIframe(frame_fname)
    arg = arg+ta.getSetWorkFrame(frame_fname)
    rtn = ta.getJavascript(arg, div_id).gsub(/div/, 'span')
   else
    ta  = TestlinkAdapter.new
    rtn = ta.getTestlinkIframe(obj, args, true)
   end
    rtn

  end
end

